@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2021-2024 Tetsuya Isaki
+ * Copyright (C) 2021-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,7 +30,6 @@
 
 #include "common.h"
 #include "image_priv.h"
-#include <errno.h>
 #include <string.h>
 
 struct image_reductor_handle_;
@@ -67,11 +66,18 @@ typedef struct image_reductor_handle_
 
 static uint finder_gray(image_reductor_handle *, ColorRGB);
 static uint finder_fixed8(image_reductor_handle *, ColorRGB);
-static uint finder_ansi16(image_reductor_handle *, ColorRGB);
+static uint finder_vga16(image_reductor_handle *, ColorRGB);
 static uint finder_fixed256(image_reductor_handle *, ColorRGB);
+#if defined(SIXELV)
+static uint finder_xterm256(image_reductor_handle *, ColorRGB);
+static inline uint8 finder_xterm256_channel(uint8);
+#endif
 static void colorcvt_gray(ColorRGBint32 *);
 static ColorRGB *image_alloc_gray_palette(uint);
 static ColorRGB *image_alloc_fixed256_palette(void);
+#if defined(SIXELV)
+static ColorRGB *image_alloc_xterm256_palette(void);
+#endif
 
 #if defined(SIXELV)
 static void image_reduct_simple(image_reductor_handle *,
@@ -89,7 +95,7 @@ static inline uint8 saturate_uint8(int);
 static inline int16 saturate_adderr(int16, int);
 
 static const ColorRGB palette_fixed8[];
-static const ColorRGB palette_ansi16[];
+static const ColorRGB palette_vga16[];
 
 // opt を初期化する。
 void
@@ -97,10 +103,11 @@ image_opt_init(image_opt *opt)
 {
 	opt->method  = REDUCT_HIGH_QUALITY;
 	opt->diffuse = DIFFUSE_SFL;
-	opt->color   = ReductorColor_Fixed256;
+	opt->color   = COLOR_FMT_256_RGB332;
 	opt->cdm     = 0;
 	opt->gain    = 256;
 	opt->output_ormode = false;
+	opt->output_transbg = false;
 	opt->suppress_palette = false;
 }
 
@@ -270,93 +277,149 @@ image_get_preferred_size(
 	}
 }
 
-// サポートしているローダを string で返す。表示用。
-string *
-image_get_loaderinfo(void)
-{
-	string *s = string_init();
+// ローダごとにサポートしているファイル形式をビットマップフラグにしたもの。
+#define LOADERMAP_gif	(1U << IMAGE_LOADER_GIF)
+#define LOADERMAP_jpeg	(1U << IMAGE_LOADER_JPEG)
+#define LOADERMAP_png	(1U << IMAGE_LOADER_PNG)
+#define LOADERMAP_webp	(1U << IMAGE_LOADER_WEBP)
+#define LOADERMAP_stb	((1U << IMAGE_LOADER_BMP)	| \
+						 (1U << IMAGE_LOADER_GIF)	| \
+						 (1U << IMAGE_LOADER_JPEG)	| \
+						 (1U << IMAGE_LOADER_PNG)	| \
+						 (1U << IMAGE_LOADER_PNM))
 
-#define ADD(name)	do {	\
-	if (string_len(s) != 0)	\
-		string_append_cstr(s, ", ");	\
-	string_append_cstr(s, name);	\
-} while (0)
-
-	// ここはアルファベット順。
-	ADD("blurhash");
-#if defined(USE_LIBPNG)
-	ADD("libpng");
-#endif
+// サポートしているローダ。処理順に並べること。
+static const struct {
+	image_match_t match;
+	image_read_t  read;
+	const char *libname;	// image_get_loaderinfo() で表示する名前
+	const char *name;		// マクロ展開用とデバッグログとかで使う短縮名
+	uint32 supported;		// このローダがサポートしている画像形式
+} loader[] = {
+#define ENTRY(name, libname, map)	\
+	{ image_##name##_match, image_##name##_read, \
+	  #libname, #name, map }
 #if defined(USE_LIBWEBP)
-	ADD("libwebp");
+	ENTRY(webp, libwebp, LOADERMAP_webp),
+#endif
+#if defined(USE_LIBJPEG)
+	ENTRY(jpeg, libjpeg, LOADERMAP_jpeg),
+#endif
+#if defined(USE_LIBPNG)
+	ENTRY(png, libpng, LOADERMAP_png),
+#endif
+#if defined(USE_GIFLIB)
+	ENTRY(gif, giflib, LOADERMAP_gif),
 #endif
 #if defined(USE_STB_IMAGE)
-	ADD("stb_image");
+	ENTRY(stb, stb_image, LOADERMAP_stb),
 #endif
+#undef ENTRY
+};
 
-	return s;
+// サポートしているローダの一覧を返す。
+// 戻り値は char * の配列で、{ name1, loader1, name2, loader2, ... }
+// のように IMAGE_LOADER_* の画像形式とそのローダ名がペアで並んでおり、
+// NULL で終端。
+// 戻り値は使い終わったら free() すること。
+char **
+image_get_loaderinfo(void)
+{
+	static const char *names[] = { IMAGE_LOADER_NAMES };
+	char **dst;
+	char **d;
+
+	dst = calloc(IMAGE_LOADER_MAX * 2 + 1, sizeof(char *));
+	if (dst == NULL) {
+		return NULL;
+	}
+	d = dst;
+
+	for (uint n = 0; n < IMAGE_LOADER_MAX; n++) {
+		uint32 filetype = (1U << n);
+		int i;
+		for (i = 0; i < countof(loader); i++) {
+			if ((loader[i].supported & filetype)) {
+				break;
+			}
+		}
+		const char *lib = NULL;
+		if (i < countof(loader)) {
+			lib = UNCONST(loader[i].libname);
+		} else if (n == IMAGE_LOADER_BLURHASH) {
+			// Blurhash は loader[] テーブルには出てこないので手動処理。
+			lib = "builtin";
+		} else {
+			continue;
+		}
+		*d++ = UNCONST(names[n]);
+		*d++ = UNCONST(lib);
+	}
+
+	// 終端は1つ。
+	*d = NULL;
+
+	return dst;
+}
+
+// pstream の画像形式を判定する。
+// 判定出来れば、image_read() に渡すための非負のインデックスを返す。
+// 判定出来なければ -1 を返す。
+// ここでは Blurhash は扱わない。
+int
+image_match(struct pstream *ps, const struct diag *diag)
+{
+	FILE *fp;
+	int idx = -1;
+
+	fp = pstream_open_for_peek(ps);
+	if (fp == NULL) {
+		Debug(diag, "pstream_open_for_peek() failed");
+		return -1;
+	}
+
+	if (countof(loader) == 0) {
+		Debug(diag, "%s: no decoders available", __func__);
+		goto done;
+	}
+
+	for (uint i = 0; i < countof(loader); i++) {
+		int ok = loader[i].match(fp, diag);
+		Trace(diag, "Checking %-4s.. %s",
+			loader[i].name, (ok ? "matched" : "no"));
+		fseek(fp, 0, SEEK_SET);
+		if (ok) {
+			idx = i;
+			goto done;
+		}
+	}
+	Trace(diag, "%s: unsupported image format", __func__);
+
+ done:
+	fclose(fp);
+	return idx;
 }
 
 // pstream から画像を読み込んで image を作成して返す。
-// 読み込めなければ errno をセットして NULL を返す。
-// 戻り値 NULL で errno = 0 なら画像形式を認識できなかったことを示す。
-// ここでは Blurhash は扱わない。
+// idx は image_match() で返された非負の値を指定すること。
+// axis, width, height はリサイズ用のヒントで、これを使うかどうかは
+// 画像ローダによる (今の所 jpeg のみ)。
+// デコードに失敗すると NULL を返す。
 struct image *
-image_read_pstream(struct pstream *ps, const struct diag *diag)
+image_read(struct pstream *ps, int idx, const image_read_hint *hint,
+	const struct diag *diag)
 {
-	int ok = -1;
-	FILE *pfp;
 	FILE *fp;
 
-	pfp = pstream_open_for_peek(ps);
-	if (pfp == NULL) {
-		Debug(diag, "pstream_open_for_peek() failed");
+	fp = pstream_open_for_read(ps);
+	if (fp == NULL) {
+		Debug(diag, "%s: pstream_open_for_read() failed", __func__);
 		return NULL;
 	}
+	struct image *img = loader[idx].read(fp, hint, diag);
+	fclose(fp);
 
-	static const struct {
-		image_match_t match;
-		image_read_t  read;
-		const char *name;
-	} loader[] = {
-#define ENTRY(name)	{ image_##name##_match, image_##name##_read, #name }
-#if defined(USE_LIBWEBP)
-		ENTRY(webp),
-#endif
-#if defined(USE_LIBPNG)
-		ENTRY(png),
-#endif
-#if defined(USE_STB_IMAGE)
-		ENTRY(stb),
-#endif
-#undef ENTRY
-	};
-	for (uint i = 0; i < countof(loader); i++) {
-		ok = loader[i].match(pfp, diag);
-		Trace(diag, "Checking %-4s.. %s",
-			loader[i].name, (ok ? "matched" : "no"));
-		fseek(pfp, 0, SEEK_SET);
-		if (ok) {
-			fclose(pfp);
-			fp = pstream_open_for_read(ps);
-			if (fp == NULL) {
-				Debug(diag, "%s: pstream_open_for_read() failed", __func__);
-				return NULL;
-			}
-			struct image *img = loader[i].read(fp, diag);
-			fclose(fp);
-			return img;
-		}
-	}
-
-	if (ok < 0) {
-		Debug(diag, "%s: no decoders available", __func__);
-	} else {
-		Trace(diag, "%s: unsupported image format", __func__);
-	}
-
-	errno = 0;
-	return NULL;
+	return img;
 }
 
 // 入力画像を 16bit 内部形式にインプレース変換する。
@@ -422,10 +485,10 @@ image_reduct(
 	dst->has_alpha = src->has_alpha;
 
 	// 減色モードからパレットオペレーションを用意。
-	switch (opt->color & ReductorColor_MASK) {
-	 case ReductorColor_Gray:
+	switch (opt->color & COLOR_FMT_MASK) {
+	 case COLOR_FMT_GRAY:
 	 {
-		uint graycount = ((uint)opt->color >> 8) + 1;
+		uint graycount = GET_COLOR_COUNT(opt->color);
 		ir->palette_buf = image_alloc_gray_palette(graycount);
 		if (ir->palette_buf == NULL) {
 			goto abort;
@@ -437,19 +500,19 @@ image_reduct(
 		break;
 	 }
 
-	 case ReductorColor_Fixed8:
+	 case COLOR_FMT_8_RGB:
 		ir->finder  = finder_fixed8;
 		ir->palette = palette_fixed8;
 		ir->palette_count = 8;
 		break;
 
-	 case ReductorColor_ANSI16:
-		ir->finder  = finder_ansi16;
-		ir->palette = palette_ansi16;
+	 case COLOR_FMT_16_VGA:
+		ir->finder  = finder_vga16;
+		ir->palette = palette_vga16;
 		ir->palette_count = 16;
 		break;
 
-	 case ReductorColor_Fixed256:
+	 case COLOR_FMT_256_RGB332:
 		ir->palette_buf = image_alloc_fixed256_palette();
 		if (ir->palette_buf == NULL) {
 			goto abort;
@@ -459,6 +522,18 @@ image_reduct(
 		ir->finder = finder_fixed256;
 		break;
 
+#if defined(SIXELV)
+	 case COLOR_FMT_256_XTERM:
+		ir->palette_buf = image_alloc_xterm256_palette();
+		if (ir->palette_buf == NULL) {
+			goto abort;
+		}
+		ir->palette = ir->palette_buf;
+		ir->palette_count = 256;
+		ir->finder = finder_xterm256;
+		break;
+#endif
+
 	 default:
 		Debug(diag, "%s: Unsupported color 0x%x", __func__, opt->color);
 		goto abort;
@@ -974,10 +1049,10 @@ finder_fixed8(image_reductor_handle *ir, ColorRGB c)
 #endif
 }
 
-// ANSI 固定 16 色。
-// Standard VGA colors を基準とし、
+// VGA 固定 16 色。
+// ANSI16 の Standard VGA colors を基準とし、
 // ただしパレット4 を Brown ではなく Yellow になるようにしてある。
-static const ColorRGB palette_ansi16[] = {
+static const ColorRGB palette_vga16[] = {
 	{ RGBToU32(  0,   0,   0) },
 	{ RGBToU32(170,   0,   0) },
 	{ RGBToU32(  0, 170,   0) },
@@ -996,9 +1071,9 @@ static const ColorRGB palette_ansi16[] = {
 	{ RGBToU32(255, 255, 255) },
 };
 
-// 色 c を ANSI 固定 16 色パレットへ変換する。
+// 色 c を VGA 固定 16 色パレットへ変換する。
 static uint
-finder_ansi16(image_reductor_handle *ir, ColorRGB c)
+finder_vga16(image_reductor_handle *ir, ColorRGB c)
 {
 	uint R;
 	uint G;
@@ -1063,9 +1138,75 @@ finder_fixed256(image_reductor_handle *ir, ColorRGB c)
 	return (R << 5) | (G << 2) | B;
 }
 
-
 #if defined(SIXELV)
 
+// xterm 互換の固定 256 色パレットを作成して返す。
+static ColorRGB *
+image_alloc_xterm256_palette(void)
+{
+	ColorRGB *pal = malloc(sizeof(ColorRGB) * 256);
+	int i;
+
+	if (pal == NULL) {
+		return NULL;
+	}
+	// ANSI16 色。
+	memcpy(pal, palette_vga16, sizeof(palette_vga16));
+
+	// 216色 (6x6x6)。
+	for (i = 0; i < 216; i++) {
+		ColorRGB c;
+
+		// レベルは 00, 5f, 87, af, d7, ff で、00 だけ直線上にない。
+		c.r = ((i / (6*6)) % 6);
+		c.r = c.r == 0 ? 0 : c.r * 0x28 + 0x37;
+		c.g = ((i / (6)  ) % 6);
+		c.g = c.g == 0 ? 0 : c.g * 0x28 + 0x37;
+		c.b = ((i        ) % 6);
+		c.b = c.b == 0 ? 0 : c.b * 0x28 + 0x37;
+		pal[i + 16] = c;
+	}
+
+	// グレー24色。
+	for (i = 0; i < 24; i++) {
+		ColorRGB c;
+		c.r = c.g = c.b = 8 + i * 10;
+		pal[i + 16 + 216] = c;
+	}
+
+	return pal;
+}
+
+// xterm256色の1チャンネル分、0 .. 5 を返す。
+static inline uint8
+finder_xterm256_channel(uint8 c)
+{
+	// レベル: 00, 5f, 87, af, d7, ff
+	// しきい:   2f, 73, 9b, bc, eb
+
+	if (c < 0x73) {
+		if (c < 0x2f) {
+			c = 0;
+		} else {
+			c = 1;
+		}
+	} else {
+		c = 2 + (c - 0x73) / 0x28;
+	}
+	return c;
+}
+
+// xterm 互換 256 色で c に最も近いパレット番号を返す。
+static uint
+finder_xterm256(image_reductor_handle *ir, ColorRGB c)
+{
+	return 16
+		+ finder_xterm256_channel(c.r) * 36
+		+ finder_xterm256_channel(c.g) * 6
+		+ finder_xterm256_channel(c.b) * 1;
+}
+
+
 //
 // enum のデバッグ表示用
 //
@@ -1134,32 +1275,32 @@ reductordiffuse_tostr(ReductorDiffuse diffuse)
 	return buf;
 }
 
-// ReductorColor を文字列にする。
+// ColorFormat を文字列にする。
 // (内部バッファを使う可能性があるため同時に2回呼ばないこと)
 const char *
-reductorcolor_tostr(ReductorColor color)
+colorformat_tostr(ColorFormat color)
 {
 	static const struct {
-		ReductorColor value;
+		ColorFormat value;
 		const char *name;
 	} table[] = {
-		{ ReductorColor_Gray,		"Gray" },
-		{ ReductorColor_Fixed8,		"Fixed8" },
-		{ ReductorColor_ANSI16,		"ANSI16" },
-		{ ReductorColor_Fixed256,	"Fixed256" },
+		{ COLOR_FMT_GRAY,		"Gray" },
+		{ COLOR_FMT_8_RGB,		"8(RGB)" },
+		{ COLOR_FMT_16_VGA,		"16(ANSI VGA)" },
+		{ COLOR_FMT_256_RGB332,	"256(RGB332)" },
+		{ COLOR_FMT_256_XTERM,	"256(xterm)" },
 	};
 	static char buf[16];
-	uint type = (uint)color & ReductorColor_MASK;
-	uint num = (uint)color >> 8;
+	uint type = (uint)color & COLOR_FMT_MASK;
 
 	for (int i = 0; i < countof(table); i++) {
-		if (type == table[i].value) {
-			// 今のところ Gray は必ず num > 0 なのでこれだけで区別できる
-			if (num == 0) {
-				return table[i].name;
-			} else {
+		if (table[i].value == type) {
+			if (type == COLOR_FMT_GRAY) {
+				uint num = GET_COLOR_COUNT(color);
 				snprintf(buf, sizeof(buf), "%s%u", table[i].name, num);
 				return buf;
+			} else {
+				return table[i].name;
 			}
 		}
 	}
