@@ -1,6 +1,6 @@
 /* vi:set ts=4: */
 /*
- * Copyright (C) 2014-2024 Tetsuya Isaki
+ * Copyright (C) 2014-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,8 +40,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#define SAYAKA_VERSION "3.8.1"
-#define SAYAKA_RELDATE "2024/09/21"
+#define SAYAKA_VERSION "3.8.2"
+#define SAYAKA_RELDATE "2025/05/17"
 
 enum {
 	DEFAULT_FONT_WIDTH	= 7,
@@ -240,32 +240,32 @@ main(int ac, char *av[])
 			int n;
 			if (strcmp(optarg, "1") == 0) {
 				colormode = 1;
-				imageopt.color = ReductorColor_GrayLevel(2);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(2);
 				imageopt.cdm   = 96;
 			} else if (strcmp(optarg, "2") == 0) {
 				colormode = 2;
-				imageopt.color = ReductorColor_GrayLevel(2);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(2);
 				imageopt.cdm   = 96;
 			} else if (strcmp(optarg, "8") == 0) {
 				colormode = 8;
-				imageopt.color = ReductorColor_Fixed8;
+				imageopt.color = COLOR_FMT_8_RGB;
 				imageopt.cdm   = 96;
 			} else if (strcmp(optarg, "16") == 0) {
 				colormode = 16;
-				imageopt.color = ReductorColor_ANSI16;
+				imageopt.color = COLOR_FMT_16_VGA;
 			} else if (strcmp(optarg, "256") == 0) {
 				colormode = 256;
-				imageopt.color = ReductorColor_Fixed256;
+				imageopt.color = COLOR_FMT_256_RGB332;
 			} else if (strcmp(optarg, "gray") == 0 ||
 			           strcmp(optarg, "grey") == 0) {
 				colormode = 2;
-				imageopt.color = ReductorColor_GrayLevel(256);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(256);
 			} else if ((strncmp(optarg, "gray", 4) == 0 ||
 			            strncmp(optarg, "grey", 4) == 0   ) &&
 			           (n = stou32def(optarg + 4, -1, NULL)) != (uint32)-1 &&
 			           (2 <= n && n <= 256)) {
 				colormode = 2;
-				imageopt.color = ReductorColor_GrayLevel(n);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(n);
 			} else {
 				errx(1, "%s: invalid color mode", optarg);
 			}
@@ -668,7 +668,7 @@ mkdir_if(const char *dirname)
 		if (r < 0) {
 			err(1, "%s: mkdir %s", __func__, dirname);
 		}
-		printf("%s: create %s\n", __func__, dirname);
+		warnx("create %s", dirname);
 	}
 }
 
@@ -692,11 +692,7 @@ init_screen(void)
 	// 端末の背景色を調べる。
 	// 判定できなければ背景色白をデフォルトにしておく。
 	// モノクロモードなら不要。
-	if (opt_bgtheme == BG_AUTO && is_tty &&
-		(imageopt.color == ReductorColor_Fixed8 ||
-		 imageopt.color == ReductorColor_ANSI16 ||
-		 imageopt.color == ReductorColor_Fixed256 ))
-	{
+	if (opt_bgtheme == BG_AUTO && is_tty && colormode > 2) {
 		progress("Checking background color...");
 		opt_bgtheme = terminal_get_bgtheme();
 		switch (opt_bgtheme) {
@@ -743,10 +739,10 @@ init_screen(void)
 	init_color();
 
 	// キャッシュファイル用の色モード名。
-	switch (imageopt.color & ReductorColor_MASK) {
-	 case ReductorColor_Gray:
+	switch (imageopt.color & COLOR_FMT_MASK) {
+	 case COLOR_FMT_GRAY:
 	 {
-		uint grayscale = (imageopt.color >> 8) + 1;
+		uint grayscale = GET_COLOR_COUNT(imageopt.color);
 		if (grayscale == 2) {
 			strlcpy(colorname, "2", sizeof(colorname));
 		} else {
@@ -754,13 +750,13 @@ init_screen(void)
 		}
 		break;
 	 }
-	 case ReductorColor_Fixed8:
+	 case COLOR_FMT_8_RGB:
 		strlcpy(colorname, "8", sizeof(colorname));
 		break;
-	 case ReductorColor_ANSI16:
+	 case COLOR_FMT_16_VGA:
 		strlcpy(colorname, "16", sizeof(colorname));
 		break;
-	 case ReductorColor_Fixed256:
+	 case COLOR_FMT_256_RGB332:
 		strlcpy(colorname, "256", sizeof(colorname));
 		break;
 	 default:
@@ -788,7 +784,7 @@ invalidate_cache(void)
 
 	// 写真は2日分くらいか。
 	snprintf(cmd, sizeof(cmd),
-		"find %s -name http\\* -type f -atime +2 -exec rm {} +", cachedir);
+		"find %s -name file\\* -type f -atime +2 -exec rm {} +", cachedir);
 	if (system(cmd) < 0) {
 		warn("system(find photo)");
 	}
