@@ -1,7 +1,7 @@
 /* vi:set ts=4: */
 /*
  * Copyright (C) 2015 Y.Sugahara (moveccr)
- * Copyright (C) 2021-2024 Tetsuya Isaki
+ * Copyright (C) 2021-2025 Tetsuya Isaki
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,7 +32,6 @@
 #include "sixelv.h"
 #include "image.h"
 #include <err.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
 #include <signal.h>
@@ -40,8 +39,8 @@
 #include <unistd.h>
 #include <sys/time.h>
 
-#define SIXELV_VERSION "3.8.1"
-#define SIXELV_RELDATE "2024/09/21"
+#define SIXELV_VERSION "3.8.2"
+#define SIXELV_RELDATE "2025/05/17"
 
 typedef enum {
 	OUTPUT_FORMAT_SIXEL,
@@ -49,14 +48,17 @@ typedef enum {
 } OutputFormat;
 
 static void version(void);
+static void list(void);
 static void usage(void);
 static void help_all(void);
 static bool do_file(const char *filename);
+static struct image *read_blurhash(struct pstream *, uint *, uint *);
 static void signal_handler(int);
 
 static struct diag *diag_image;
 static struct diag *diag_net;
 static struct diag *diag_sixel;
+static bool show_filename;			// 画像の前にファイル名を表示
 static bool ignore_error;			// true ならエラーでも次ファイルを処理
 static FILE *ofp;					// 出力中のストリーム
 static bool opt_blurhash_nearest;	// Blurhash を最近傍補間する
@@ -91,10 +93,12 @@ enum {
 	OPT_help_all,
 	OPT_ipv4,
 	OPT_ipv6,
+	OPT_list,
 	OPT_output_format,
 	OPT_profile,
 	OPT_resize_axis,
 	OPT_sixel_or,
+	OPT_sixel_transbg,
 	OPT_suppress_palette,
 	OPT_version,
 	OPT_width,
@@ -117,13 +121,15 @@ static const struct option longopts[] = {
 	{ "ignore-error",	no_argument,		NULL,	'i' },
 	{ "ipv4",			no_argument,		NULL,	OPT_ipv4 },
 	{ "ipv6",			no_argument,		NULL,	OPT_ipv6 },
+	{ "list",			no_argument,		NULL,	OPT_list },
 	{ "output-format",	required_argument,	NULL,	'O' },
 	{ "profile",		no_argument,		NULL,	OPT_profile },
 	{ "reduction",		required_argument,	NULL,	'r' },
 	{ "resize-axis",	required_argument,	NULL,	OPT_resize_axis },
 	{ "sixel-or",		no_argument,		NULL,	OPT_sixel_or },
+	{ "sixel-transbg",	no_argument,		NULL,	OPT_sixel_transbg },
 	{ "suppress-palette", no_argument,		NULL,	OPT_suppress_palette },
-	{ "version",		no_argument,		NULL,	'v' },
+	{ "version",		no_argument,		NULL,	OPT_version },
 	{ "width",			required_argument,	NULL,	'w' },
 	{ NULL },
 };
@@ -213,21 +219,23 @@ main(int ac, char *av[])
 		 {
 			int n;
 			if (strcmp(optarg, "2") == 0) {
-				imageopt.color = ReductorColor_GrayLevel(2);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(2);
 			} else if (strcmp(optarg, "8") == 0) {
-				imageopt.color = ReductorColor_Fixed8;
+				imageopt.color = COLOR_FMT_8_RGB;
 			} else if (strcmp(optarg, "16") == 0) {
-				imageopt.color = ReductorColor_ANSI16;
+				imageopt.color = COLOR_FMT_16_VGA;
 			} else if (strcmp(optarg, "256") == 0) {
-				imageopt.color = ReductorColor_Fixed256;
+				imageopt.color = COLOR_FMT_256_RGB332;
+			} else if (strcmp(optarg, "xterm256") == 0) {
+				imageopt.color = COLOR_FMT_256_XTERM;
 			} else if (strcmp(optarg, "gray") == 0 ||
 			           strcmp(optarg, "grey") == 0) {
-				imageopt.color = ReductorColor_GrayLevel(256);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(256);
 			} else if ((strncmp(optarg, "gray", 4) == 0 ||
 			            strncmp(optarg, "grey", 4) == 0   ) &&
 			           (n = stou32def(optarg + 4, -1, NULL)) != (uint32)-1 &&
 			           (2 <= n && n <= 256)) {
-				imageopt.color = ReductorColor_GrayLevel(n);
+				imageopt.color = MAKE_COLOR_FMT_GRAY(n);
 			} else {
 				errx(1, "%s: invalid color mode", optarg);
 			}
@@ -311,6 +319,10 @@ main(int ac, char *av[])
 			netopt.address_family = 6;
 			break;
 
+		 case OPT_list:
+			list();
+			exit(0);
+
 		 case 'O':
 			output_format = parse_optmap(map_output_format, optarg);
 			if ((int)output_format < 0) {
@@ -348,11 +360,19 @@ main(int ac, char *av[])
 			imageopt.output_ormode = true;
 			break;
 
+		 case OPT_sixel_transbg:
+			imageopt.output_transbg = true;
+			break;
+
 		 case OPT_suppress_palette:
 			imageopt.suppress_palette = true;
 			break;
 
 		 case 'v':
+			show_filename = true;
+			break;
+
+		 case OPT_version:
 			version();
 			exit(0);
 
@@ -394,6 +414,9 @@ main(int ac, char *av[])
 			infilename = NULL;
 		} else {
 			infilename = av[i];
+			if (show_filename) {
+				printf("%s\n", infilename);
+			}
 		}
 		if (do_file(infilename) == false && ignore_error == false) {
 			rv = 1;
@@ -407,12 +430,17 @@ main(int ac, char *av[])
 static void
 version(void)
 {
-	string *info = image_get_loaderinfo();
-
 	printf("%s %s (%s) - SIXEL viewer\n", progname, progver, SIXELV_RELDATE);
-	printf(" Supported loader: %s\n", string_get(info));
+}
 
-	string_free(info);
+static void
+list(void)
+{
+	char **names = image_get_loaderinfo();
+	for (uint i = 0; names[i] != NULL; i += 2) {
+		printf("%-8s %s\n", names[i], names[i + 1]);
+	}
+	free(names);
 }
 
 static void
@@ -427,13 +455,27 @@ usage(void)
 "  -r <method>     : Reduction method, none(simple) or high (default:high)\n"
 "  -O <fmt>        : Output format, bmp or sixel (default: sixel)\n"
 "  -o <filename>   : Output filename, '-' means stdout (default: -)\n"
-"  -d <diffusion>                        --resize-axis=<axis>\n"
-"  --gain=<gain>                         --blurhash-nearest\n"
-"  --sixel-or                            --suppress-palette\n"
-"  --ignore-error                        --ciphers=<ciphers>\n"
-"  --help-all                            --debug-image=<0..2>\n"
-"  --debug-net=<0..2>                    --debug-sixel=<0..2>\n"
-	);
+"  -v              : Show input filename\n");
+
+	static const char * const opts[] = {
+		"-d <diffusion>",
+		"--resize-axis=<axis>",
+		"--gain=<gain>",
+		"--blurhash-nearest",
+		"--sixel-or",
+		"--sixel-transbg",
+		"--suppress-palette",
+		"--ignore-error",
+		"--ciphers=<ciphers>",
+		"--help-all",
+		"--debug-image=<0..2>",
+		"--debug-net=<0..2>",
+		"--debug-sixel=<0..2>",
+		"", // 偶数パディング
+	};
+	for (uint i = 0; i < countof(opts) / 2; i++) {
+		fprintf(stderr, "  %-39s%s\n", opts[i * 2], opts[i * 2 + 1]);
+	}
 }
 
 static void
@@ -444,6 +486,7 @@ help_all(void)
 	fprintf(stderr,
 "  -c,--color=<colormode> : Set color mode (default:256)\n"
 "     256      : Fixed 256 colors (MSX SCREEN8 compatible palette)\n"
+"     xterm256 : Fixed 256 colors (xterm compatible palette)\n"
 "     16       : Fixed ANSI compatible 16 colors\n"
 "     8        : Fixed RGB 8 colors\n"
 "     2        : Monochrome (2-level grayscale)\n"
@@ -472,14 +515,17 @@ help_all(void)
 "                           between 0.0 and 1.0 (default:1.0)\n"
 "  --gain=<gain>          : Set output gain between 0.0 and 2.0 (default:1.0)\n"
 "  --help-all             : This help\n"
+"  --list                 : Show supported filetype and decoder list\n"
 "  -O,--output-format=<fmt> : bmp or sixel (default:sixel)\n"
 "  -o <filename>          : Output filename, '-' means stdout (default:-)\n"
 "  --sixel-or             : Output SIXEL by OR-mode\n"
+"  --sixel-transbg        : Make SIXEL background transparent\n"
 "  --suppress-palette     : Suppress output of SIXEL palette definition\n"
 "  --ciphers <ciphers>    : \"RSA\" can only be specified\n"
 "  --ipv4 / --ipv6        : Connect only IPv4/v6\n"
 "  -i,--ignore-error\n"
-"  -v,--version\n"
+"  -v                     : Show input filename\n"
+"  --version\n"
 "  --debug-image=<0..2>\n"
 "  --debug-net  =<0..2>\n"
 "  --debug-sixel=<0..2>\n"
@@ -500,6 +546,7 @@ do_file(const char *infile)
 	int ifd = -1;
 	FILE *ifp = NULL;
 	const char *infilename;	// 表示用
+	image_read_hint hint;
 	uint dst_width;
 	uint dst_height;
 	struct timeval load_start;
@@ -567,95 +614,40 @@ do_file(const char *infile)
 
 	PROF(&load_start);
 
-	// 読み込み。
-	srcimg = image_read_pstream(pstream, diag_image);
-	if (srcimg) {
-		// 得られた画像サイズと引数指定から、いい感じにサイズを決定。
-		image_get_preferred_size(srcimg->width, srcimg->height,
-			opt_resize_axis, opt_width, opt_height,
-			&dst_width, &dst_height);
-	} else {
-		// 読み込めなければ Blurhash を試す。
-
-		// 先に生成する画像サイズを決定してローダに渡す必要がある。
-		// -w,-h	--bn
-		// なし		なし		: 20倍で生成、等倍にリサイズ。
-		// あり		なし		: WxH で生成、等倍にリサイズ。
-		// なし		あり		: 1倍で生成、 20倍にリサイズ。
-		// あり		あり		: 1倍で生成、 WxH にリサイズ。
-
-		int bw;
-		int bh;
-		if (opt_blurhash_nearest) {
-			bw = -1;
-			bh = -1;
-		} else {
-			if (opt_width == 0 && opt_height == 0) {
-				// -w, -h ともに指定されなければ勝手に縦横 20倍とする。
-				bw = -20;
-				bh = -20;
-			} else if (opt_width > 0 && opt_height > 0) {
-				// 両方指定されたらそのサイズ。
-				bw = opt_width;
-				bh = opt_height;
-			} else {
-				// -w か -h 片方しか指定されなかった場合、どのみち
-				// オリジナルのアスペクト比も不明なので 1:1 とするしかない。
-				if (opt_width > 0) {
-					bw = opt_width;
-				} else {
-					bw = opt_height;
-				}
-				bh = bw;
-			}
-		}
-
-		FILE *fp = pstream_open_for_read(pstream);
-		if (fp) {
-			srcimg = image_blurhash_read(fp, bw, bh, diag_image);
-			fclose(fp);
-		}
+	// 画像形式判定。
+	int loader_idx = image_match(pstream, diag_image);
 
+	if (loader_idx >= 0) {
+		// 読み込み。
+		// (この hint は libjpeg の scaling hint のことで、これをもとに
+		// 1/8 とかで読み込んだものが srcimg->{width,height} になる)
+		memset(&hint, 0, sizeof(hint));
+		hint.axis   = opt_resize_axis;
+		hint.width  = opt_width;
+		hint.height = opt_height;
+		srcimg = image_read(pstream, loader_idx, &hint, diag_image);
 		if (srcimg) {
-			// リサイズ後のサイズを決定。
-			if (opt_blurhash_nearest) {
-				if (opt_width == 0 && opt_height == 0) {
-					dst_width  = srcimg->width * 20;
-					dst_height = srcimg->height * 20;
-				} else if (opt_width > 0 && opt_height > 0) {
-					dst_width  = opt_width;
-					dst_height = opt_height;
-				} else {
-					if (opt_width > 0) {
-						dst_width = opt_width;
-					} else {
-						dst_width = opt_height;
-					}
-					dst_height = dst_width;
-				}
-			} else {
-				// 等倍にリサイズ。
-				dst_width  = srcimg->width;
-				dst_height = srcimg->height;
-			}
+			// 得られた画像サイズと引数指定から、いい感じにサイズを決定。
+			image_get_preferred_size(srcimg->width, srcimg->height,
+				opt_resize_axis, opt_width, opt_height,
+				&dst_width, &dst_height);
 		}
+	} else {
+		// どの画像形式でもなさそうなら Blurhash を試す。
+		srcimg = read_blurhash(pstream, &dst_width, &dst_height);
 	}
 
 	PROF(&load_end);
 
 	if (srcimg == NULL) {
-		if (errno == 0) {
-			warnx("%s: Unknown image format", infilename);
-		} else {
-			warn("%s: image_read_pstream() failed", infilename);
-		}
+		warnx("%s: Unknown image format", infilename);
 		goto abort;
 	}
 
 	Debug(diag_image,
 		"InputSize=(%u, %u) OutputSize=(%u, %u) OutputColor=%s",
 		srcimg->width, srcimg->height, dst_width, dst_height,
-		reductorcolor_tostr(imageopt.color));
+		colorformat_tostr(imageopt.color));
 
 	PROF(&cvt_start);
 	image_convert_to16(srcimg);
@@ -742,6 +734,88 @@ do_file(const char *infile)
 	return rv;
 }
 
+// pstream から Blurhash 画像を読み込んで返す。
+// その際画像とコマンドラインオプションから求めた表示画像サイズを
+// dst_width, dst_height に返す。
+static struct image *
+read_blurhash(struct pstream *pstream, uint *dst_width, uint *dst_height)
+{
+	FILE *fp;
+	struct image *srcimg;
+	int bw;
+	int bh;
+	uint width;
+	uint height;
+
+	// 先に生成する画像サイズを決定してローダに渡す必要がある。
+	// -w,-h	--bn
+	// なし		なし		: 20倍で生成、等倍にリサイズ。
+	// あり		なし		: WxH で生成、等倍にリサイズ。
+	// なし		あり		: 1倍で生成、 20倍にリサイズ。
+	// あり		あり		: 1倍で生成、 WxH にリサイズ。
+
+	if (opt_blurhash_nearest) {
+		bw = -1;
+		bh = -1;
+	} else {
+		if (opt_width == 0 && opt_height == 0) {
+			// -w, -h ともに指定されなければ勝手に縦横 20倍とする。
+			bw = -20;
+			bh = -20;
+		} else if (opt_width > 0 && opt_height > 0) {
+			// 両方指定されたらそのサイズ。
+			bw = opt_width;
+			bh = opt_height;
+		} else {
+			// -w か -h 片方しか指定されなかった場合、どのみち
+			// オリジナルのアスペクト比も不明なので 1:1 とするしかない。
+			if (opt_width > 0) {
+				bw = opt_width;
+			} else {
+				bw = opt_height;
+			}
+			bh = bw;
+		}
+	}
+
+	srcimg = NULL;
+
+	fp = pstream_open_for_read(pstream);
+	if (fp == NULL) {
+		return NULL;
+	}
+
+	srcimg = image_blurhash_read(fp, bw, bh, diag_image);
+	fclose(fp);
+
+	if (srcimg) {
+		// リサイズ後のサイズを決定。
+		if (opt_blurhash_nearest) {
+			if (opt_width == 0 && opt_height == 0) {
+				width  = srcimg->width * 20;
+				height = srcimg->height * 20;
+			} else if (opt_width > 0 && opt_height > 0) {
+				width  = opt_width;
+				height = opt_height;
+			} else {
+				if (opt_width > 0) {
+					width = opt_width;
+				} else {
+					width = opt_height;
+				}
+				height = width;
+			}
+		} else {
+			// 等倍にリサイズ。
+			width  = srcimg->width;
+			height = srcimg->height;
+		}
+		*dst_width  = width;
+		*dst_height = height;
+	}
+	return srcimg;
+}
+
 static void
 signal_handler(int signo)
 {
